@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 #include <infinoted/infinoted-options.h>
 #include <infinoted/infinoted-util.h>
 #include <libinfinity/inf-i18n.h>
+#include <libinfinity/common/inf-protocol.h>
 
 #ifdef LIBINFINITY_HAVE_LIBDAEMON
 # include <libdaemon/dpid.h>
@@ -29,6 +30,7 @@
 
 #include <glib.h>
 #include <stdlib.h>
+#include <signal.h>
 #include <string.h>
 #include <errno.h>
 
@@ -204,6 +206,37 @@ infinoted_options_key_file_get_string(GKeyFile* keyfile,
   return infinoted_options_propagate_key_file_error(error, local_error);
 }
 
+static gboolean
+infinoted_options_key_file_get_string_list(GKeyFile* keyfile,
+                                           const gchar* keyname,
+                                           gchar*** result,
+                                           GError** error)
+{
+  GError* local_error;
+  gchar** ret;
+
+  local_error = NULL;
+  ret = g_key_file_get_string_list(
+    keyfile,
+    INFINOTED_OPTIONS_GROUP,
+    keyname,
+    NULL,
+    &local_error
+  );
+
+  if(local_error == NULL)
+  {
+    g_strfreev(*result);
+    *result = ret;
+  }
+  else
+  {
+    g_strfreev(ret);
+  }
+
+  return infinoted_options_propagate_key_file_error(error, local_error);
+}
+
 static gboolean
 infinoted_options_key_file_get_integer(GKeyFile* keyfile,
                                        const gchar* keyname,
@@ -259,6 +292,7 @@ infinoted_options_load_key_file(const GOptionEntry* entries,
   gboolean result;
   gchar* string;
   gchar* filename;
+  gchar** string_list;
 
   for(entry = entries; entry->long_name != NULL; ++ entry)
   {
@@ -310,6 +344,33 @@ infinoted_options_load_key_file(const GOptionEntry* entries,
           }
         }
 
+        break;
+      case G_OPTION_ARG_STRING_ARRAY:
+        string_list = NULL;
+
+        result = infinoted_options_key_file_get_string_list(
+          key_file,
+          entry->long_name,
+          &string_list,
+          error
+        );
+
+        if(result == TRUE)
+        {
+          /* Can return TRUE without having string list set, for example in
+           * case the key is not set at all, in which case we just don't
+           * overwrite the existing value. */
+          if(entry->arg_data && string_list != NULL)
+          {
+            g_strfreev(*(gchar***)entry->arg_data);
+            *(gchar***)entry->arg_data = string_list;
+          }
+          else
+          {
+            g_strfreev(string_list);
+          }
+        }
+
         break;
       case G_OPTION_ARG_FILENAME:
         string = NULL;
@@ -398,16 +459,50 @@ infinoted_options_validate(InfinotedOptions* options,
                            GError** error)
 {
   InfXmppConnectionSecurityPolicy security_policy;
+  gboolean requires_password;
+
   security_policy = options->security_policy;
 
-  if(options->password != NULL &&
+#ifdef LIBINFINITY_HAVE_PAM
+  if(options->password != NULL && options->pam_service != NULL)
+  {
+    g_set_error(
+      error,
+      infinoted_options_error_quark(),
+      INFINOTED_OPTIONS_ERROR_INVALID_AUTHENTICATION_SETTINGS,
+      "%s",
+      _("Cannot use both server password and system authentication.")
+    );
+    return FALSE;
+  }
+
+  if(options->pam_service == NULL
+     && (options->pam_allowed_users != NULL
+         || options->pam_allowed_groups != NULL))
+  {
+    g_set_error(
+      error,
+      infinoted_options_error_quark(),
+      INFINOTED_OPTIONS_ERROR_INVALID_AUTHENTICATION_SETTINGS,
+      "%s",
+      _("Need a pam service to authenticate users.")
+    );
+    return FALSE;
+  }
+#endif /* LIBINFINITY_HAVE_PAM */
+
+  requires_password = options->password != NULL;
+#ifdef LIBINFINITY_HAVE_PAM
+  requires_password = requires_password || options->pam_service != NULL;
+#endif /* LIBINFINITY_HAVE_PAM */
+
+  if(requires_password &&
      options->security_policy == INF_XMPP_CONNECTION_SECURITY_ONLY_UNSECURED)
   {
     infinoted_util_log_warning(
       _("Requiring password through unencrypted connection."));
   }
 
-
   if(options->create_key == TRUE && options->create_certificate == FALSE)
   {
     g_set_error(
@@ -543,6 +638,20 @@ infinoted_options_load(InfinotedOptions* options,
     { "password", 'P', 0,
       G_OPTION_ARG_STRING, NULL,
       N_("Require given password on connections"), N_("PASSWORD") },
+#ifdef LIBINFINITY_HAVE_PAM
+    { "pam-service", 0, 0,
+      G_OPTION_ARG_STRING, NULL,
+      N_("Authenticate clients against given pam service on connection"),
+      N_("SERVICE") },
+    { "allow-user", 0, 0,
+      G_OPTION_ARG_STRING_ARRAY, NULL,
+      N_("User allowed to connect after pam authentication"),
+      N_("USER") },
+    { "allow-group", 0, 0,
+      G_OPTION_ARG_STRING_ARRAY, NULL,
+      N_("Group allowed to connect after pam authentication"),
+      N_("GROUP") },
+#endif /* LIBINFINITY_HAVE_PAM */
     { "sync-directory", 0, 0,
       G_OPTION_ARG_FILENAME, NULL,
       N_("A directory into which to periodically store a copy of the "
@@ -580,6 +689,11 @@ infinoted_options_load(InfinotedOptions* options,
   entries[i++].arg_data = &options->root_directory;
   entries[i++].arg_data = &autosave_interval;
   entries[i++].arg_data = &options->password;
+#ifdef LIBINFINITY_HAVE_PAM
+  entries[i++].arg_data = &options->pam_service;
+  entries[i++].arg_data = &options->pam_allowed_users;
+  entries[i++].arg_data = &options->pam_allowed_groups;
+#endif /* LIBINFINITY_HAVE_PAM */
   entries[i++].arg_data = &options->sync_directory;
   entries[i++].arg_data = &sync_interval;
 #ifdef LIBINFINITY_HAVE_LIBDAEMON
@@ -635,18 +749,16 @@ infinoted_options_load(InfinotedOptions* options,
     {
       g_free(security_policy);
 
-      if(!infinoted_util_set_daemon_pid_file_proc(error))
-        return FALSE;
-
-#ifdef DAEMON_PID_FILE_KILL_WAIT_AVAILABLE
-      if(daemon_pid_file_kill_wait(SIGTERM, 5) != 0)
-#else
-      if(daemon_pid_file_kill(SIGTERM) != 0)
-#endif
+      infinoted_util_daemon_set_global_pid_file_proc();
+      if(infinoted_util_daemon_pid_file_kill(SIGTERM) != 0)
       {
-        infinoted_util_set_errno_error(error, errno,
-          _("Could not kill daemon"));
-        return FALSE;
+        infinoted_util_daemon_set_local_pid_file_proc();
+        if(infinoted_util_daemon_pid_file_kill(SIGTERM) != 0)
+        {
+          infinoted_util_set_errno_error(error, errno,
+            _("Could not kill daemon"));
+          return FALSE;
+        }
       }
 
       exit(0);
@@ -697,6 +809,30 @@ infinoted_options_load(InfinotedOptions* options,
     options->password = NULL;
   }
 
+#ifdef LIBINFINITY_HAVE_PAM
+  if(options->pam_service != NULL && strcmp(options->pam_service, "") == 0)
+  {
+    g_free(options->pam_service);
+    options->pam_service = NULL;
+  }
+
+  /* treat it as undefining the option if only one entry, which is empty,
+   * is given */
+  if(options->pam_allowed_users != NULL
+     && strcmp(options->pam_allowed_users[0], "") == 0
+     && options->pam_allowed_users[1] == NULL) {
+    g_free(options->pam_allowed_users[0]);
+    g_free(options->pam_allowed_users);
+  }
+
+  if(options->pam_allowed_groups != NULL
+     && strcmp(options->pam_allowed_groups[0], "") == 0
+     && options->pam_allowed_groups[1] == NULL) {
+    g_free(options->pam_allowed_groups[0]);
+    g_free(options->pam_allowed_groups);
+  }
+#endif /* LIBINFINITY_HAVE_PAM */
+
   if(options->sync_directory != NULL &&
      strcmp(options->sync_directory, "") == 0)
   {
@@ -742,12 +878,17 @@ infinoted_options_new(const gchar* const* config_files,
   options->certificate_chain_file = NULL;
   options->create_key = FALSE;
   options->create_certificate = FALSE;
-  options->port = 6523;
+  options->port = inf_protocol_get_default_port();
   options->security_policy = INF_XMPP_CONNECTION_SECURITY_BOTH_PREFER_TLS;
   options->root_directory =
     g_build_filename(g_get_home_dir(), ".infinote", NULL);
   options->autosave_interval = 0;
   options->password = NULL;
+#ifdef LIBINFINITY_HAVE_PAM
+  options->pam_service = NULL;
+  options->pam_allowed_users = NULL;
+  options->pam_allowed_groups = NULL;
+#endif /* LIBINFINITY_HAVE_PAM */
   options->sync_directory = NULL;
   options->sync_interval = 0;
 
@@ -778,6 +919,11 @@ infinoted_options_free(InfinotedOptions* options)
   g_free(options->certificate_chain_file);
   g_free(options->root_directory);
   g_free(options->password);
+#ifdef LIBINFINITY_HAVE_PAM
+  g_free(options->pam_service);
+  g_strfreev(options->pam_allowed_users);
+  g_strfreev(options->pam_allowed_groups);
+#endif
   g_free(options->sync_directory);
   g_slice_free(InfinotedOptions, options);
 }
