@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,8 +21,8 @@
 #define __INF_GTK_BROWSER_MODEL_FILTER_H__
 
 #include <libinfgtk/inf-gtk-browser-model.h>
-#include <gtk/gtktreemodelfilter.h>
 
+#include <gtk/gtk.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
