@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,9 +21,8 @@
 #define __INF_GTK_BROWSER_VIEW_H__
 
 #include <libinfgtk/inf-gtk-browser-model.h>
-#include <gtk/gtkbin.h>
-#include <gtk/gtkmenu.h>
 
+#include <gtk/gtk.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
@@ -41,7 +40,7 @@ typedef struct _InfGtkBrowserView InfGtkBrowserView;
 typedef struct _InfGtkBrowserViewClass InfGtkBrowserViewClass;
 
 struct _InfGtkBrowserViewClass {
-  GtkBinClass parent_class;
+  GtkTreeViewClass parent_class;
 
   /* signals */
   void (*activate)(InfGtkBrowserView* view,
@@ -53,13 +52,15 @@ struct _InfGtkBrowserViewClass {
   void (*populate_popup)(InfGtkBrowserView* view,
                          GtkMenu* menu);
 
+#if ! GTK_CHECK_VERSION(2, 91, 0)
   void (*set_scroll_adjustments)(InfGtkBrowserView* view,
                                  GtkAdjustment* hadjustment,
                                  GtkAdjustment* vadjustment);
+#endif
 };
 
 struct _InfGtkBrowserView {
-  GtkBin parent;
+  GtkTreeView parent;
 };
 
 GType
@@ -71,9 +72,6 @@ inf_gtk_browser_view_new(void);
 GtkWidget*
 inf_gtk_browser_view_new_with_model(InfGtkBrowserModel* model);
 
-InfGtkBrowserModel*
-inf_gtk_browser_view_get_model(InfGtkBrowserView* view);
-
 gboolean
 inf_gtk_browser_view_get_selected(InfGtkBrowserView* view,
                                   GtkTreeIter* iter);
