@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -339,6 +339,7 @@ inf_gtk_certificate_manager_certificate_func(InfXmppConnection* connection,
 
   InfGtkCertificateManagerQuery* query;
   gchar* text;
+  GtkWidget* vbox;
   GtkWidget* button;
   GtkWidget* image;
   GtkWidget* label;
@@ -519,7 +520,11 @@ inf_gtk_certificate_manager_certificate_func(InfXmppConnection* connection,
       query->connection = connection;
       query->dialog = inf_gtk_certificate_dialog_new(
         priv->parent_window,
+#if GTK_CHECK_VERSION(2,90,7)
+        0,
+#else
         GTK_DIALOG_NO_SEPARATOR,
+#endif
         flags,
         hostname,
         chain
@@ -578,13 +583,14 @@ inf_gtk_certificate_manager_certificate_func(InfXmppConnection* connection,
       gtk_widget_show(label);
       g_free(text);
 
-      gtk_box_pack_start(
-        GTK_BOX(GTK_DIALOG(query->dialog)->vbox),
-        label,
-        FALSE,
-        FALSE,
-        0
-      );
+#if GTK_CHECK_VERSION(2,14,0)
+      vbox = gtk_dialog_get_content_area(GTK_DIALOG(query->dialog));
+#else
+      vbox = GTK_DIALOG(query->dialog)->vbox;
+#endif
+
+
+      gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
       text = g_strdup_printf(
         _("Trust the certificate of host %s in the future"),
@@ -603,13 +609,7 @@ inf_gtk_certificate_manager_certificate_func(InfXmppConnection* connection,
 
       g_free(text);
 
-      gtk_box_pack_start(
-        GTK_BOX(GTK_DIALOG(query->dialog)->vbox),
-        query->checkbutton,
-        FALSE,
-        FALSE,
-        0
-      );
+      gtk_box_pack_start(GTK_BOX(vbox), query->checkbutton, FALSE, FALSE, 0);
 
       /* TODO: In which cases should the checkbutton be checked by default? */
 
