@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -214,13 +214,9 @@ inf_gtk_chat_add_message(InfGtkChat* chat,
 
   g_free(time_str);
 
-  /* If the session is still being synchronized then this is
-   * a backlog message. Apply backlog tag instead. */
-  if(inf_session_get_status(INF_SESSION(priv->session)) ==
-     INF_SESSION_SYNCHRONIZING)
-  {
+  /* Always apply backlog tag if it's a backlog message */
+  if(message->flags & INF_CHAT_BUFFER_MESSAGE_BACKLOG)
     tag = priv->tag_backlog;
-  }
 
   g_object_get( G_OBJECT(priv->vadj),
     "value", &scroll_val,
@@ -277,7 +273,8 @@ inf_gtk_chat_commit_message(InfGtkChat* chat)
       /* TODO: Use gtk_entry_get_text_length() - (text -
        * gtk_entry_get_text())) once we can use GTK+ 2.14. */
       strlen(text),
-      time(NULL)
+      time(NULL),
+      0
     );
   }
   else
@@ -288,7 +285,8 @@ inf_gtk_chat_commit_message(InfGtkChat* chat)
       text,
       /* TODO: Use gtk_entry_get_text_length() once we can use GTK+ 2.14. */
       strlen(text),
-      time(NULL)
+      time(NULL),
+      0
     );
   }
 
@@ -377,7 +375,11 @@ inf_gtk_chat_entry_key_press_event_cb(GtkWidget* widget,
   /* This must not be pressed for tab completion to be triggered: */
 #define MASK (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK)
 
+#if GTK_CHECK_VERSION(2,90,7)
+  if(event->keyval == GDK_KEY_Tab && (event->state & MASK) == 0)
+#else
   if(event->keyval == GDK_Tab && (event->state & MASK) == 0)
+#endif
   {
     index = gtk_editable_get_position(GTK_EDITABLE(widget));
     if(priv->completion_text != NULL && index != priv->completion_end)
@@ -651,7 +653,7 @@ inf_gtk_chat_init(GTypeInstance* instance,
   priv->voffset = 0.0;
   priv->vmode = INF_GTK_CHAT_VMODE_DISABLED;
 
-  /* Actually there are invalid as long as completion_text is NULL, but
+  /* Actually they are invalid as long as completion_text is NULL, but
    * let's be sure */
   priv->completion_text = NULL;
   priv->completion_start = 0;
@@ -667,7 +669,11 @@ inf_gtk_chat_init(GTypeInstance* instance,
   gtk_text_view_set_indent(GTK_TEXT_VIEW(priv->chat_view), -12);
   /* TODO: this prevents copying via ctrl+c - maybe the entry ctrl+c
    * should catch this and copy from the textview instead: */
+#if GTK_CHECK_VERSION(2,18,0)
+  gtk_widget_set_can_focus(priv->chat_view, FALSE);
+#else
   GTK_WIDGET_UNSET_FLAGS(priv->chat_view, GTK_CAN_FOCUS);
+#endif
   gtk_widget_show(priv->chat_view);
 
   /* TODO: These should probably be style properties: */
