@@ -93,48 +93,9 @@ inf_marshal_VOID__BOXED_DOUBLE_DOUBLE (GClosure     *closure,
 
 /* VOID:OBJECT (inf-marshal.in:4) */
 
-/* VOID:UINT,UINT,OBJECT (inf-marshal.in:5) */
-void
-inf_marshal_VOID__UINT_UINT_OBJECT (GClosure     *closure,
-                                    GValue       *return_value G_GNUC_UNUSED,
-                                    guint         n_param_values,
-                                    const GValue *param_values,
-                                    gpointer      invocation_hint G_GNUC_UNUSED,
-                                    gpointer      marshal_data)
-{
-  typedef void (*GMarshalFunc_VOID__UINT_UINT_OBJECT) (gpointer     data1,
-                                                       guint        arg_1,
-                                                       guint        arg_2,
-                                                       gpointer     arg_3,
-                                                       gpointer     data2);
-  register GMarshalFunc_VOID__UINT_UINT_OBJECT callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
+/* VOID:POINTER (inf-marshal.in:5) */
 
-  g_return_if_fail (n_param_values == 4);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = *(GMarshalFunc_VOID__UINT_UINT_OBJECT*) (marshal_data ? &marshal_data : &cc->callback);
-
-  callback (data1,
-            g_marshal_value_peek_uint (param_values + 1),
-            g_marshal_value_peek_uint (param_values + 2),
-            g_marshal_value_peek_object (param_values + 3),
-            data2);
-}
-
-/* VOID:POINTER (inf-marshal.in:6) */
-
-/* VOID:OBJECT,DOUBLE (inf-marshal.in:7) */
+/* VOID:OBJECT,DOUBLE (inf-marshal.in:6) */
 void
 inf_marshal_VOID__OBJECT_DOUBLE (GClosure     *closure,
                                  GValue       *return_value G_GNUC_UNUSED,
@@ -171,7 +132,7 @@ inf_marshal_VOID__OBJECT_DOUBLE (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,POINTER (inf-marshal.in:8) */
+/* VOID:OBJECT,POINTER (inf-marshal.in:7) */
 void
 inf_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
@@ -208,7 +169,7 @@ inf_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:POINTER,UINT (inf-marshal.in:9) */
+/* VOID:POINTER,UINT (inf-marshal.in:8) */
 void
 inf_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                 GValue       *return_value G_GNUC_UNUSED,
@@ -245,9 +206,9 @@ inf_marshal_VOID__POINTER_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT (inf-marshal.in:10) */
+/* VOID:UINT (inf-marshal.in:9) */
 
-/* VOID:UINT,UINT (inf-marshal.in:11) */
+/* VOID:UINT,UINT (inf-marshal.in:10) */
 void
 inf_marshal_VOID__UINT_UINT (GClosure     *closure,
                              GValue       *return_value G_GNUC_UNUSED,
@@ -284,7 +245,7 @@ inf_marshal_VOID__UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,BOOLEAN (inf-marshal.in:12) */
+/* VOID:OBJECT,BOOLEAN (inf-marshal.in:11) */
 void
 inf_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
@@ -321,7 +282,7 @@ inf_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,BOXED,OBJECT (inf-marshal.in:13) */
+/* VOID:UINT,BOXED,OBJECT (inf-marshal.in:12) */
 void
 inf_marshal_VOID__UINT_BOXED_OBJECT (GClosure     *closure,
                                      GValue       *return_value G_GNUC_UNUSED,
@@ -360,24 +321,25 @@ inf_marshal_VOID__UINT_BOXED_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,INT (inf-marshal.in:14) */
+/* VOID:UINT,INT,BOOLEAN (inf-marshal.in:13) */
 void
-inf_marshal_VOID__UINT_INT (GClosure     *closure,
-                            GValue       *return_value G_GNUC_UNUSED,
-                            guint         n_param_values,
-                            const GValue *param_values,
-                            gpointer      invocation_hint G_GNUC_UNUSED,
-                            gpointer      marshal_data)
+inf_marshal_VOID__UINT_INT_BOOLEAN (GClosure     *closure,
+                                    GValue       *return_value G_GNUC_UNUSED,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint G_GNUC_UNUSED,
+                                    gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__UINT_INT) (gpointer     data1,
-                                               guint        arg_1,
-                                               gint         arg_2,
-                                               gpointer     data2);
-  register GMarshalFunc_VOID__UINT_INT callback;
+  typedef void (*GMarshalFunc_VOID__UINT_INT_BOOLEAN) (gpointer     data1,
+                                                       guint        arg_1,
+                                                       gint         arg_2,
+                                                       gboolean     arg_3,
+                                                       gpointer     data2);
+  register GMarshalFunc_VOID__UINT_INT_BOOLEAN callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 3);
+  g_return_if_fail (n_param_values == 4);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -389,15 +351,16 @@ inf_marshal_VOID__UINT_INT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = *(GMarshalFunc_VOID__UINT_INT*) (marshal_data ? &marshal_data : &cc->callback);
+  callback = *(GMarshalFunc_VOID__UINT_INT_BOOLEAN*) (marshal_data ? &marshal_data : &cc->callback);
 
   callback (data1,
             g_marshal_value_peek_uint (param_values + 1),
             g_marshal_value_peek_int (param_values + 2),
+            g_marshal_value_peek_boolean (param_values + 3),
             data2);
 }
 
-/* VOID:OBJECT,OBJECT (inf-marshal.in:15) */
+/* VOID:OBJECT,OBJECT (inf-marshal.in:14) */
 void
 inf_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                  GValue       *return_value G_GNUC_UNUSED,
@@ -434,7 +397,7 @@ inf_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,OBJECT (inf-marshal.in:16) */
+/* VOID:BOXED,OBJECT (inf-marshal.in:15) */
 void
 inf_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                 GValue       *return_value G_GNUC_UNUSED,
@@ -471,7 +434,7 @@ inf_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:BOXED,BOXED,OBJECT (inf-marshal.in:17) */
+/* VOID:BOXED,BOXED,OBJECT (inf-marshal.in:16) */
 void
 inf_marshal_VOID__BOXED_BOXED_OBJECT (GClosure     *closure,
                                       GValue       *return_value G_GNUC_UNUSED,
@@ -510,7 +473,7 @@ inf_marshal_VOID__BOXED_BOXED_OBJECT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,OBJECT,BOOLEAN (inf-marshal.in:18) */
+/* VOID:OBJECT,OBJECT,BOOLEAN (inf-marshal.in:17) */
 void
 inf_marshal_VOID__OBJECT_OBJECT_BOOLEAN (GClosure     *closure,
                                          GValue       *return_value G_GNUC_UNUSED,
@@ -549,11 +512,11 @@ inf_marshal_VOID__OBJECT_OBJECT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:DOUBLE (inf-marshal.in:19) */
+/* VOID:DOUBLE (inf-marshal.in:18) */
 
-/* VOID:ENUM (inf-marshal.in:20) */
+/* VOID:ENUM (inf-marshal.in:19) */
 
-/* BOOLEAN:OBJECT,OBJECT (inf-marshal.in:21) */
+/* BOOLEAN:OBJECT,OBJECT (inf-marshal.in:20) */
 void
 inf_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
                                     GValue       *return_value G_GNUC_UNUSED,
@@ -594,7 +557,7 @@ inf_marshal_BOOLEAN__OBJECT_OBJECT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:OBJECT,UINT (inf-marshal.in:22) */
+/* VOID:OBJECT,UINT (inf-marshal.in:21) */
 void
 inf_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                GValue       *return_value G_GNUC_UNUSED,
@@ -631,3 +594,83 @@ inf_marshal_VOID__OBJECT_UINT (GClosure     *closure,
             data2);
 }
 
+/* POINTER:POINTER (inf-marshal.in:22) */
+void
+inf_marshal_POINTER__POINTER (GClosure     *closure,
+                              GValue       *return_value G_GNUC_UNUSED,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
+                              gpointer      marshal_data)
+{
+  typedef gpointer (*GMarshalFunc_POINTER__POINTER) (gpointer     data1,
+                                                     gpointer     arg_1,
+                                                     gpointer     data2);
+  register GMarshalFunc_POINTER__POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gpointer v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 2);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = *(GMarshalFunc_POINTER__POINTER*) (marshal_data ? &marshal_data : &cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_pointer (param_values + 1),
+                       data2);
+
+  g_value_set_pointer (return_value, v_return);
+}
+
+/* POINTER:OBJECT,POINTER (inf-marshal.in:23) */
+void
+inf_marshal_POINTER__OBJECT_POINTER (GClosure     *closure,
+                                     GValue       *return_value G_GNUC_UNUSED,
+                                     guint         n_param_values,
+                                     const GValue *param_values,
+                                     gpointer      invocation_hint G_GNUC_UNUSED,
+                                     gpointer      marshal_data)
+{
+  typedef gpointer (*GMarshalFunc_POINTER__OBJECT_POINTER) (gpointer     data1,
+                                                            gpointer     arg_1,
+                                                            gpointer     arg_2,
+                                                            gpointer     data2);
+  register GMarshalFunc_POINTER__OBJECT_POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gpointer v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = *(GMarshalFunc_POINTER__OBJECT_POINTER*) (marshal_data ? &marshal_data : &cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_object (param_values + 1),
+                       g_marshal_value_peek_pointer (param_values + 2),
+                       data2);
+
+  g_value_set_pointer (return_value, v_return);
+}
+
