@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,7 @@
  * MA 02110-1301, USA.
  */
 
-#ifndef G_OS_WIN32
-/* Need this for O_NOFOLLOW and fdopendir and so on */
-# define _GNU_SOURCE
-#endif
+#include "config.h"
 
 #include <libinfinity/server/infd-filesystem-storage.h>
 #include <libinfinity/server/infd-storage.h>
@@ -153,7 +150,6 @@ infd_filesystem_storage_remove_rec(const gchar* path,
 {
 #ifdef G_OS_WIN32
   SHFILEOPSTRUCTW op;
-  gchar* from_dup;
   gunichar2* from;
   glong len;
   gboolean result;
@@ -200,7 +196,7 @@ infd_filesystem_storage_remove_rec(const gchar* path,
   ret = g_unlink(path);
   if(ret == -1)
   {
-    if(errno == EISDIR)
+    if(errno == EISDIR || errno == EPERM)
     {
       dir = g_dir_open(path, 0, error);
       if(dir == NULL) return FALSE;
@@ -324,13 +320,14 @@ infd_filesystem_storage_storage_read_subdirectory(InfdStorage* storage,
   InfdFilesystemStorage* fs_storage;
   InfdFilesystemStoragePrivate* priv;
   GSList* list;
-#ifndef G_OS_WIN32
+#if !defined(G_OS_WIN32) && !defined(__APPLE__)
   int dir_fd;
   DIR* dir;
   struct dirent* dir_entry;
   struct dirent* dir_result;
   struct stat stat_buf;
   int saved_errno;
+  enum { F_UNKNOWN, F_DIR, F_REG, F_LNK } filetype;
 #else
   GDir* dir;
   const gchar* name;
@@ -356,11 +353,12 @@ infd_filesystem_storage_storage_read_subdirectory(InfdStorage* storage,
 
   list = NULL;
 
-#ifndef G_OS_WIN32
+#if !defined(G_OS_WIN32) && !defined(__APPLE__)
   dir_fd = open(full_name, O_NOFOLLOW | O_RDONLY);
   if(dir_fd == -1 || (dir = fdopendir(dir_fd)) == NULL)
   {
     infd_filesystem_storage_system_error(errno, error);
+    if(dir_fd != -1) close(dir_fd);
     g_free(full_name);
     return NULL;
   }
@@ -380,33 +378,44 @@ infd_filesystem_storage_storage_read_subdirectory(InfdStorage* storage,
     if(converted_name != NULL && strcmp(converted_name, ".") != 0
                               && strcmp(converted_name, "..") != 0)
     {
-      /* Some filesystems, such as reiserfs, don't support reporting the
-       * entry's file type. In that case we do an additional lstat here. */
-      if(dir_result->d_type == DT_UNKNOWN)
+      filetype = F_UNKNOWN;
+
+#ifdef HAVE_D_TYPE
+      if(dir_result->d_type == DT_LNK)
+        filetype = F_LNK;
+      else if(dir_result->d_type == DT_DIR)
+        filetype = F_DIR;
+      else if(dir_result->d_type == DT_REG)
+        filetype = F_REG;
+      else if(dir_result->d_type == DT_UNKNOWN)
+#endif
       {
+        /* Some filesystems, such as reiserfs, don't support reporting the
+         * entry's file type. In that case we do an additional lstat here.
+         * Also lstat if d_type is not available on this platform. */
         file_path = g_build_filename(full_name, dir_result->d_name, NULL);
         if(lstat(file_path, &stat_buf) == 0)
         {
           if(S_ISDIR(stat_buf.st_mode))
-            dir_result->d_type = DT_DIR;
+            filetype = F_DIR;
           else if(S_ISREG(stat_buf.st_mode))
-            dir_result->d_type = DT_REG;
+            filetype = F_REG;
           else if(S_ISLNK(stat_buf.st_mode))
-            dir_result->d_type = DT_LNK;
+            filetype = F_LNK;
         }
         g_free(file_path);
       }
 
-      if(dir_result->d_type != DT_LNK && dir_result->d_type != DT_UNKNOWN)
+      if(filetype != F_LNK && filetype != F_UNKNOWN)
       {
-        if(dir_result->d_type == DT_DIR)
+        if(filetype == F_DIR)
         {
           list = g_slist_prepend(
             list,
             infd_storage_node_new_subdirectory(converted_name)
           );
         }
-        else if(dir_result->d_type == DT_REG)
+        else if(filetype == F_REG)
         {
           /* The note type identifier is behind the last '.' */
           separator = g_strrstr_len(converted_name, name_len, ".");
@@ -426,10 +435,18 @@ infd_filesystem_storage_storage_read_subdirectory(InfdStorage* storage,
   }
 
   g_free(dir_entry);
+  if(closedir(dir) == -1)
+  {
+    infd_filesystem_storage_system_error(errno, error);
+    infd_storage_node_list_free(list);
+    g_free(full_name);
+    return NULL;
+  }
 
   if(saved_errno != 0)
   {
     infd_filesystem_storage_system_error(saved_errno, error);
+    infd_storage_node_list_free(list);
     g_free(full_name);
     return NULL;
   }
