@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007, 2008, 2009 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,6 +22,7 @@
 #include <libinfinity/common/inf-ip-address.h>
 #include <libinfinity/common/inf-io.h>
 #include <libinfinity/inf-marshal.h>
+#include <config.h>
 
 #ifndef G_OS_WIN32
 # include <sys/types.h>
@@ -52,6 +53,7 @@
 typedef struct _InfdTcpServerPrivate InfdTcpServerPrivate;
 struct _InfdTcpServerPrivate {
   InfIo* io;
+  InfIoWatch* watch;
 
   InfNativeSocket socket;
   InfdTcpServerStatus status;
@@ -183,6 +185,12 @@ infd_tcp_server_io(InfNativeSocket* socket,
   InfTcpConnection* connection;
   GError* error;
 
+  union {
+    struct sockaddr in_generic;
+    struct sockaddr_in in;
+    struct sockaddr_in6 in6;
+  } native_addr;
+
   server = INFD_TCP_SERVER(user_data);
   priv = INFD_TCP_SERVER_PRIVATE(server);
   g_object_ref(G_OBJECT(server));
@@ -202,7 +210,18 @@ infd_tcp_server_io(InfNativeSocket* socket,
   {
     do
     {
-      new_socket = accept(priv->socket, NULL, NULL);
+      /* Note that we do not do anything with native_addr and len. This is
+       * currently only for debugging purposes since there seemingly was a
+       * case where accept returned a valid but socket and errno was set to
+       * EAGAIN (which might have been resulted from a previous call however
+       * since the errno = 0 line was not there yet before that either).
+       * I hope to get some more information this way in case this occurs
+       * again. */
+#ifndef G_OS_WIN32
+      errno = 0;
+#endif
+      len = sizeof(native_addr);
+      new_socket = accept(priv->socket, &native_addr.in_generic, &len);
       errcode = INFD_TCP_SERVER_LAST_ERROR;
 
       if(new_socket == INVALID_SOCKET &&
@@ -383,14 +402,9 @@ infd_tcp_server_error(InfdTcpServer* server,
 
   if(priv->status == INFD_TCP_SERVER_OPEN)
   {
-    inf_io_watch(
-      priv->io,
-      &priv->socket,
-      0,
-      infd_tcp_server_io,
-      server,
-      NULL
-    );
+    g_assert(priv->watch != NULL);
+    inf_io_remove_watch(priv->io, priv->watch);
+    priv->watch = NULL;
   }
 
   if(priv->socket != INVALID_SOCKET)
@@ -600,6 +614,10 @@ infd_tcp_server_bind(InfdTcpServer* server,
   struct sockaddr* addr;
   socklen_t addrlen;
 
+#if !defined(G_OS_WIN32) && defined(HAVE_SO_REUSEADDR)
+  int value;
+#endif
+
   g_return_val_if_fail(INFD_IS_TCP_SERVER(server), FALSE);
   priv = INFD_TCP_SERVER_PRIVATE(server);
 
@@ -661,6 +679,21 @@ infd_tcp_server_bind(InfdTcpServer* server,
     return FALSE;
   }
 
+#if !defined(G_OS_WIN32) && defined(HAVE_SO_REUSEADDR)
+  /* Allow fast restarts of servers by enabling SO_REUSEADDR */
+  value = 1;
+
+  if(setsockopt(priv->socket, SOL_SOCKET, SO_REUSEADDR, &value,
+      sizeof(int)) == -1)
+  {
+    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+
+    closesocket(priv->socket);
+    priv->socket = INVALID_SOCKET;
+    return FALSE;
+  }
+#endif
+
   if(bind(priv->socket, addr, addrlen) == -1)
   {
     infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
@@ -727,10 +760,12 @@ infd_tcp_server_open(InfdTcpServer* server,
                      GError** error)
 {
   InfdTcpServerPrivate* priv;
-  int result;
+  gboolean was_bound;
 
 #ifdef G_OS_WIN32
   u_long argp;
+#else
+  int result;
 #endif
 
   g_return_val_if_fail(INFD_IS_TCP_SERVER(server), FALSE);
@@ -738,10 +773,12 @@ infd_tcp_server_open(InfdTcpServer* server,
 
   g_return_val_if_fail(priv->io != NULL, FALSE);
   g_return_val_if_fail(priv->status != INFD_TCP_SERVER_OPEN, FALSE);
+  g_assert(priv->watch == NULL);
 
   g_object_freeze_notify(G_OBJECT(server));
 
-  if(priv->status == INFD_TCP_SERVER_CLOSED)
+  was_bound = (priv->status != INFD_TCP_SERVER_CLOSED);
+  if(!was_bound)
   {
     if(!infd_tcp_server_bind(server, error))
     {
@@ -750,14 +787,13 @@ infd_tcp_server_open(InfdTcpServer* server,
     }
   }
 
-  /* TODO: If something fails here, and we were already bound previously,
-   * then don't fallback to closed. */
 #ifndef G_OS_WIN32
   result = fcntl(priv->socket, F_GETFL);
   if(result == -1)
   {
     infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
-    infd_tcp_server_close(server);
+    if(!was_bound)
+      infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
     return FALSE;
   }
@@ -765,7 +801,8 @@ infd_tcp_server_open(InfdTcpServer* server,
   if(fcntl(priv->socket, F_SETFL, result | O_NONBLOCK) == -1)
   {
     infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
-    infd_tcp_server_close(server);
+    if(!was_bound)
+      infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
     return FALSE;
   }
@@ -774,7 +811,8 @@ infd_tcp_server_open(InfdTcpServer* server,
   if(ioctlsocket(priv->socket, FIONBIO, &argp) != 0)
   {
     infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
-    infd_tcp_server_close(server);
+    if(!was_bound)
+      infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
     return FALSE;
   }
@@ -783,12 +821,13 @@ infd_tcp_server_open(InfdTcpServer* server,
   if(listen(priv->socket, 5) == -1)
   {
     infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
-    infd_tcp_server_close(server);
+    if(!was_bound)
+      infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
     return FALSE;
   }
 
-  inf_io_watch(
+  priv->watch = inf_io_add_watch(
     priv->io,
     &priv->socket,
     INF_IO_INCOMING | INF_IO_ERROR,
@@ -823,14 +862,9 @@ infd_tcp_server_close(InfdTcpServer* server)
 
   if(priv->status == INFD_TCP_SERVER_OPEN)
   {
-    inf_io_watch(
-      priv->io,
-      &priv->socket,
-      0,
-      infd_tcp_server_io,
-      server,
-      NULL
-    );
+    g_assert(priv->watch != NULL);
+    inf_io_remove_watch(priv->io, priv->watch);
+    priv->watch = NULL;
   }
 
   closesocket(priv->socket);
